@@ -41,6 +41,8 @@ class arma_ostream
   
   template<typename eT> inline static std::streamsize modify_stream(std::ostream& o, const eT*              data, const uword n_elem);
   template<typename  T> inline static std::streamsize modify_stream(std::ostream& o, const std::complex<T>* data, const uword n_elem);
+  template<typename eT> inline static std::streamsize modify_stream(std::ostream& o, typename SpMat<eT>::const_iterator begin, const uword n_elem, const typename arma_not_cx<eT>::result* junk = 0);
+  template<typename  T> inline static std::streamsize modify_stream(std::ostream& o, typename SpMat<T>::const_iterator begin, const uword n_elem, const typename arma_cx_only<T>::result* junk = 0);
   
   template<typename eT> inline static void print_elem_zero(std::ostream& o);
   
@@ -52,6 +54,10 @@ class arma_ostream
   
   template<typename oT> inline static void print(std::ostream& o, const field<oT>&         m);
   template<typename oT> inline static void print(std::ostream& o, const subview_field<oT>& m);
+
+
+  template<typename eT> inline static void print_dense(std::ostream& o, const SpMat<eT>& m, const bool modify);
+  template<typename eT> inline static void print(std::ostream& o, const SpMat<eT>& m, const bool modify);
   };
 
 
