@@ -316,11 +316,11 @@ accu(const T1& x)
     }
   else
     {
-    typename SpProxy<T1>::const_iterator_type it     = p.begin();
+    typename SpProxy<T1>::const_iterator_type it = p.begin();
     
     eT result = eT(0);
     
-    while(it.pos() < p.get_n_nonzero())
+    while(it != p.end())
       {
       result += (*it);
       ++it;
