@@ -16,13 +16,12 @@
 
 
 
-
-//! for two arrays, generic version
+//! for two arrays, generic version for non-complex values
 template<typename eT>
 arma_hot
 arma_pure
 arma_inline
-eT
+typename arma_not_cx<eT>::result
 op_dot::direct_dot_arma(const uword n_elem, const eT* const A, const eT* const B)
   {
   arma_extra_debug_sigprint();
@@ -48,6 +47,41 @@ op_dot::direct_dot_arma(const uword n_elem, const eT* const A, const eT* const B
 
 
 
+//! for two arrays, generic version for complex values
+template<typename eT>
+arma_hot
+arma_pure
+inline
+typename arma_cx_only<eT>::result
+op_dot::direct_dot_arma(const uword n_elem, const eT* const A, const eT* const B)
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename get_pod_type<eT>::result T;
+  
+  T val_real = T(0);
+  T val_imag = T(0);
+  
+  for(uword i=0; i<n_elem; ++i)
+    {
+    const std::complex<T>& X = A[i];
+    const std::complex<T>& Y = B[i];
+    
+    const T a = X.real();
+    const T b = X.imag();
+    
+    const T c = Y.real();
+    const T d = Y.imag();
+    
+    val_real += (a*c) - (b*d);
+    val_imag += (a*d) + (b*c);
+    }
+  
+  return std::complex<T>(val_real, val_imag);
+  }
+
+
+
 //! for two arrays, float and double version
 template<typename eT>
 arma_hot
@@ -58,7 +92,7 @@ op_dot::direct_dot(const uword n_elem, const eT* const A, const eT* const B)
   {
   arma_extra_debug_sigprint();
   
-  if( n_elem <= (128/sizeof(eT)) )
+  if( n_elem <= 32u )
     {
     return op_dot::direct_dot_arma(n_elem, A, B);
     }
@@ -94,22 +128,30 @@ arma_pure
 typename arma_cx_only<eT>::result
 op_dot::direct_dot(const uword n_elem, const eT* const A, const eT* const B)
   {
-  #if defined(ARMA_USE_ATLAS)
-    {
-    arma_extra_debug_print("atlas::cx_cblas_dot()");
-    
-    return atlas::cx_cblas_dot(n_elem, A, B);
-    }
-  #elif defined(ARMA_USE_BLAS)
-    {
-    // TODO: work out the mess with zdotu() and zdotu_sub() in BLAS
-    return op_dot::direct_dot_arma(n_elem, A, B);
-    }
-  #else
+  if( n_elem <= 16u )
     {
     return op_dot::direct_dot_arma(n_elem, A, B);
     }
-  #endif
+  else
+    {
+    #if defined(ARMA_USE_ATLAS)
+      {
+      arma_extra_debug_print("atlas::cx_cblas_dot()");
+      
+      return atlas::cx_cblas_dot(n_elem, A, B);
+      }
+    #elif defined(ARMA_USE_BLAS)
+      {
+      arma_extra_debug_print("blas::dot()");
+      
+      return blas::dot(n_elem, A, B);
+      }
+    #else
+      {
+      return op_dot::direct_dot_arma(n_elem, A, B);
+      }
+    #endif
+    }
   }
 
 
@@ -152,9 +194,9 @@ op_dot::direct_dot(const uword n_elem, const eT* const A, const eT* const B, con
 
 template<typename T1, typename T2>
 arma_hot
-arma_inline
+inline
 typename T1::elem_type
-op_dot::apply(const Base<typename T1::elem_type,T1>& X, const Base<typename T1::elem_type,T2>& Y)
+op_dot::apply(const T1& X, const T2& Y)
   {
   arma_extra_debug_sigprint();
   
@@ -172,16 +214,16 @@ op_dot::apply(const Base<typename T1::elem_type,T1>& X, const Base<typename T1::
 
 template<typename T1, typename T2>
 arma_hot
-arma_inline
+inline
 typename T1::elem_type
-op_dot::apply_unwrap(const Base<typename T1::elem_type,T1>& X, const Base<typename T1::elem_type,T2>& Y)
+op_dot::apply_unwrap(const T1& X, const T2& Y)
   {
   arma_extra_debug_sigprint();
   
   typedef typename T1::elem_type eT;
   
-  const unwrap<T1> tmp1(X.get_ref());
-  const unwrap<T2> tmp2(Y.get_ref());
+  const unwrap<T1> tmp1(X);
+  const unwrap<T2> tmp2(Y);
   
   const Mat<eT>& A = tmp1.M;
   const Mat<eT>& B = tmp2.M;
@@ -196,8 +238,8 @@ op_dot::apply_unwrap(const Base<typename T1::elem_type,T1>& X, const Base<typena
 template<typename T1, typename T2>
 arma_hot
 inline
-typename T1::elem_type
-op_dot::apply_proxy(const Base<typename T1::elem_type,T1>& X, const Base<typename T1::elem_type,T2>& Y)
+typename arma_not_cx<typename T1::elem_type>::result
+op_dot::apply_proxy(const T1& X, const T2& Y)
   {
   arma_extra_debug_sigprint();
   
@@ -205,18 +247,19 @@ op_dot::apply_proxy(const Base<typename T1::elem_type,T1>& X, const Base<typenam
   typedef typename Proxy<T1>::ea_type ea_type1;
   typedef typename Proxy<T2>::ea_type ea_type2;
   
-  const Proxy<T1> A(X.get_ref());
-  const Proxy<T2> B(Y.get_ref());
-  
   const bool prefer_at_accessor = (Proxy<T1>::prefer_at_accessor) && (Proxy<T2>::prefer_at_accessor);
   
   if(prefer_at_accessor == false)
     {
-    arma_debug_check( (A.get_n_elem() != B.get_n_elem()), "dot(): objects must have the same number of elements" );
-  
-    const uword    N  = A.get_n_elem();
-          ea_type1 PA = A.get_ea();
-          ea_type2 PB = B.get_ea();
+    const Proxy<T1> PA(X);
+    const Proxy<T2> PB(Y);
+    
+    const uword N = PA.get_n_elem();
+    
+    arma_debug_check( (N != PB.get_n_elem()), "dot(): objects must have the same number of elements" );
+    
+    ea_type1 A = PA.get_ea();
+    ea_type2 B = PB.get_ea();
     
     eT val1 = eT(0);
     eT val2 = eT(0);
@@ -225,20 +268,76 @@ op_dot::apply_proxy(const Base<typename T1::elem_type,T1>& X, const Base<typenam
     
     for(i=0, j=1; j<N; i+=2, j+=2)
       {
-      val1 += PA[i] * PB[i];
-      val2 += PA[j] * PB[j];
+      val1 += A[i] * B[i];
+      val2 += A[j] * B[j];
       }
     
     if(i < N)
       {
-      val1 += PA[i] * PB[i];
+      val1 += A[i] * B[i];
       }
     
     return val1 + val2;
     }
   else
     {
-    return op_dot::apply_unwrap(A.Q, B.Q);
+    return op_dot::apply_unwrap(X,Y);
+    }
+  }
+
+
+
+template<typename T1, typename T2>
+arma_hot
+inline
+typename arma_cx_only<typename T1::elem_type>::result
+op_dot::apply_proxy(const T1& X, const T2& Y)
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename T1::elem_type            eT;
+  typedef typename get_pod_type<eT>::result  T;
+  
+  typedef typename Proxy<T1>::ea_type ea_type1;
+  typedef typename Proxy<T2>::ea_type ea_type2;
+  
+  const bool prefer_at_accessor = (Proxy<T1>::prefer_at_accessor) && (Proxy<T2>::prefer_at_accessor);
+  
+  if(prefer_at_accessor == false)
+    {
+    const Proxy<T1> PA(X);
+    const Proxy<T2> PB(Y);
+    
+    const uword N = PA.get_n_elem();
+    
+    arma_debug_check( (N != PB.get_n_elem()), "dot(): objects must have the same number of elements" );
+    
+    ea_type1 A = PA.get_ea();
+    ea_type2 B = PB.get_ea();
+    
+    T val_real = T(0);
+    T val_imag = T(0);
+    
+    for(uword i=0; i<N; ++i)
+      {
+      const std::complex<T> X = A[i];
+      const std::complex<T> Y = B[i];
+      
+      const T a = X.real();
+      const T b = X.imag();
+      
+      const T c = Y.real();
+      const T d = Y.imag();
+      
+      val_real += (a*c) - (b*d);
+      val_imag += (a*d) + (b*c);
+      }
+    
+    return std::complex<T>(val_real, val_imag);
+    }
+  else
+    {
+    return op_dot::apply_unwrap(X,Y);
     }
   }
 
@@ -289,7 +388,7 @@ template<typename T1, typename T2>
 arma_hot
 inline
 typename T1::elem_type
-op_norm_dot::apply(const Base<typename T1::elem_type,T1>& X, const Base<typename T1::elem_type,T2>& Y)
+op_norm_dot::apply(const T1& X, const T2& Y)
   {
   arma_extra_debug_sigprint();
   
@@ -301,14 +400,15 @@ op_norm_dot::apply(const Base<typename T1::elem_type,T1>& X, const Base<typename
   
   if(prefer_at_accessor == false)
     {
-    const Proxy<T1> A(X.get_ref());
-    const Proxy<T2> B(Y.get_ref());
+    const Proxy<T1> PA(X);
+    const Proxy<T2> PB(Y);
     
-    arma_debug_check( (A.get_n_elem() != B.get_n_elem()), "norm_dot(): objects must have the same number of elements" );
+    const uword N  = PA.get_n_elem();
     
-    const uword    N  = A.get_n_elem();
-          ea_type1 PA = A.get_ea();
-          ea_type2 PB = B.get_ea();
+    arma_debug_check( (N != PB.get_n_elem()), "norm_dot(): objects must have the same number of elements" );
+    
+    ea_type1 A = PA.get_ea();
+    ea_type2 B = PB.get_ea();
     
     eT acc1 = eT(0);
     eT acc2 = eT(0);
@@ -316,14 +416,14 @@ op_norm_dot::apply(const Base<typename T1::elem_type,T1>& X, const Base<typename
     
     for(uword i=0; i<N; ++i)
       {
-      const eT tmpA = PA[i];
-      const eT tmpB = PB[i];
+      const eT tmpA = A[i];
+      const eT tmpB = B[i];
       
       acc1 += tmpA * tmpA;
       acc2 += tmpB * tmpB;
       acc3 += tmpA * tmpB;
       }
-      
+    
     return acc3 / ( std::sqrt(acc1 * acc2) );
     }
   else
@@ -338,14 +438,14 @@ template<typename T1, typename T2>
 arma_hot
 inline
 typename T1::elem_type
-op_norm_dot::apply_unwrap(const Base<typename T1::elem_type,T1>& X, const Base<typename T1::elem_type,T2>& Y)
+op_norm_dot::apply_unwrap(const T1& X, const T2& Y)
   {
   arma_extra_debug_sigprint();
   
   typedef typename T1::elem_type eT;
   
-  const unwrap<T1> tmp1(X.get_ref());
-  const unwrap<T2> tmp2(Y.get_ref());
+  const unwrap<T1> tmp1(X);
+  const unwrap<T2> tmp2(Y);
   
   const Mat<eT>& A = tmp1.M;
   const Mat<eT>& B = tmp2.M;
@@ -382,54 +482,188 @@ op_norm_dot::apply_unwrap(const Base<typename T1::elem_type,T1>& X, const Base<t
 
 
 
-template<typename T1, typename T2>
+template<typename eT>
 arma_hot
-arma_inline
-typename T1::elem_type
-op_cdot::apply(const Base<typename T1::elem_type,T1>& X, const Base<typename T1::elem_type,T2>& Y)
+arma_pure
+inline
+eT
+op_cdot::direct_cdot_arma(const uword n_elem, const eT* const A, const eT* const B)
   {
   arma_extra_debug_sigprint();
   
-  typedef typename T1::elem_type      eT;
+  typedef typename get_pod_type<eT>::result T;
+  
+  T val_real = T(0);
+  T val_imag = T(0);
+  
+  for(uword i=0; i<n_elem; ++i)
+    {
+    const std::complex<T>& X = A[i];
+    const std::complex<T>& Y = B[i];
+    
+    const T a = X.real();
+    const T b = X.imag();
+    
+    const T c = Y.real();
+    const T d = Y.imag();
+    
+    val_real += (a*c) + (b*d);
+    val_imag += (a*d) - (b*c);
+    }
+  
+  return std::complex<T>(val_real, val_imag);
+  }
+
+
+
+template<typename eT>
+arma_hot
+arma_pure
+inline
+eT
+op_cdot::direct_cdot(const uword n_elem, const eT* const A, const eT* const B)
+  {
+  arma_extra_debug_sigprint();
+  
+  if( n_elem <= 32u )
+    {
+    return op_cdot::direct_cdot_arma(n_elem, A, B);
+    }
+  else
+    {
+    #if defined(ARMA_USE_BLAS)
+      {
+      arma_extra_debug_print("blas::gemv()");
+      
+      // using gemv() workaround due to compatibility issues with cdotc() and zdotc()
+      
+      const char trans   = 'C';
+      
+      const blas_int m   = blas_int(n_elem);
+      const blas_int n   = 1;
+      //const blas_int lda = (n_elem > 0) ? blas_int(n_elem) : blas_int(1);
+      const blas_int inc = 1;
+      
+      const eT alpha     = eT(1);
+      const eT beta      = eT(0);
+      
+      eT result[2];  // paranoia: using two elements instead of one
+      
+      //blas::gemv(&trans, &m, &n, &alpha, A, &lda, B, &inc, &beta, &result[0], &inc);
+      blas::gemv(&trans, &m, &n, &alpha, A, &m, B, &inc, &beta, &result[0], &inc);
+      
+      return result[0];
+      }
+    #elif defined(ARMA_USE_ATLAS)
+      {
+      // TODO: use dedicated atlas functions cblas_cdotc_sub() and cblas_zdotc_sub() and retune threshold
+
+      return op_cdot::direct_cdot_arma(n_elem, A, B);
+      }
+    #else
+      {
+      return op_cdot::direct_cdot_arma(n_elem, A, B);
+      }
+    #endif
+    }
+  }
+
+
+
+template<typename T1, typename T2>
+arma_hot
+inline
+typename T1::elem_type
+op_cdot::apply(const T1& X, const T2& Y)
+  {
+  arma_extra_debug_sigprint();
+  
+  if( (is_Mat<T1>::value == true) && (is_Mat<T2>::value == true) )
+    {
+    return op_cdot::apply_unwrap(X,Y);
+    }
+  else
+    {
+    return op_cdot::apply_proxy(X,Y);
+    }
+  }
+
+
+
+template<typename T1, typename T2>
+arma_hot
+inline
+typename T1::elem_type
+op_cdot::apply_unwrap(const T1& X, const T2& Y)
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename T1::elem_type eT;
+  
+  const unwrap<T1> tmp1(X);
+  const unwrap<T2> tmp2(Y);
+  
+  const Mat<eT>& A = tmp1.M;
+  const Mat<eT>& B = tmp2.M;
+  
+  arma_debug_check( (A.n_elem != B.n_elem), "cdot(): objects must have the same number of elements" );
+  
+  return op_cdot::direct_cdot( A.n_elem, A.mem, B.mem );
+  }
+
+
+
+template<typename T1, typename T2>
+arma_hot
+inline
+typename T1::elem_type
+op_cdot::apply_proxy(const T1& X, const T2& Y)
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename T1::elem_type            eT;
+  typedef typename get_pod_type<eT>::result  T;
+  
   typedef typename Proxy<T1>::ea_type ea_type1;
   typedef typename Proxy<T2>::ea_type ea_type2;
   
-  const Proxy<T1> A(X.get_ref());
-  const Proxy<T2> B(Y.get_ref());
-  
   const bool prefer_at_accessor = (Proxy<T1>::prefer_at_accessor) || (Proxy<T2>::prefer_at_accessor);
   
   if(prefer_at_accessor == false)
     {
-    arma_debug_check( (A.get_n_elem() != B.get_n_elem()), "cdot(): objects must have the same number of elements" );
+    const Proxy<T1> PA(X);
+    const Proxy<T2> PB(Y);
     
-    const uword    N  = A.get_n_elem();
-          ea_type1 PA = A.get_ea();
-          ea_type2 PB = B.get_ea();
+    const uword N = PA.get_n_elem();
     
-    eT val1 = eT(0);
-    eT val2 = eT(0);
+    arma_debug_check( (N != PB.get_n_elem()), "cdot(): objects must have the same number of elements" );
     
-    uword i,j;
-    for(i=0, j=1; j<N; i+=2, j+=2)
+    ea_type1 A = PA.get_ea();
+    ea_type2 B = PB.get_ea();
+    
+    T val_real = T(0);
+    T val_imag = T(0);
+    
+    for(uword i=0; i<N; ++i)
       {
-      val1 += std::conj(PA[i]) * PB[i];
-      val2 += std::conj(PA[j]) * PB[j];
+      const std::complex<T> AA = A[i];
+      const std::complex<T> BB = B[i];
+      
+      const T a = AA.real();
+      const T b = AA.imag();
+      
+      const T c = BB.real();
+      const T d = BB.imag();
+      
+      val_real += (a*c) + (b*d);
+      val_imag += (a*d) - (b*c);
       }
     
-    if(i < N)
-      {
-      val1 += std::conj(PA[i]) * PB[i];
-      }
-    
-    return val1 + val2;
+    return std::complex<T>(val_real, val_imag);
     }
   else
     {
-    const unwrap< typename Proxy<T1>::stored_type > tmp_A(A.Q);
-    const unwrap< typename Proxy<T2>::stored_type > tmp_B(B.Q);
-    
-    return op_cdot::apply( tmp_A.M, tmp_B.M );
+    return op_cdot::apply_unwrap( X, Y );
     }
   }
 
